@@ -2,23 +2,25 @@
 """Test dlmodule.c
    Roger E. Masse  revised strategy by Barry Warsaw
 """
-from test.test_support import verbose,TestSkipped, import_module
+import unittest
+from test.test_support import verbose, import_module
 dl = import_module('dl', deprecated=True)
 
 import dircache
 libcNames = []
 libcName = ''
 for file in dircache.listdir('/usr/lib/'):
-	if '/libc.so.' in file:
-		libcNames.append(file)
+       if '/libc.so.' in file:
+               libcNames.append(file)
 # Pick the most recent (highest number) libc
 if len(libcNames) > 0:
-	libcName = sorted(libcNames)[-1]
+       libcName = sorted(libcNames)[-1]
 if not libcName:
-	raise TestSkipped
+       raise TestSkipped
+
 
 sharedlibs = [
-	(libcName, 'getpid'),
+        (libcName, 'getpid'),
     ]
 
 def test_main():
@@ -40,7 +42,7 @@ def test_main():
                 print 'worked!'
             break
     else:
-        raise TestSkipped, 'Could not open any shared libraries'
+        raise unittest.SkipTest, 'Could not open any shared libraries'
 
 
 if __name__ == '__main__':
